@@ -28,7 +28,7 @@
  * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  * 
- * $Id: channel.c 23916 2007-05-04 23:00:20Z jilles $
+ * $Id: channel.c 27011 2010-03-30 19:46:44Z leeh $
  */
 #include "stdinc.h"
 #include "rserv.h"
@@ -485,6 +485,7 @@ c_topic(struct client *client_p, const char *parv[], int parc)
 	{
 		chptr->topic[0] = '\0';
 		chptr->topicwho[0] = '\0';
+		chptr->topic_tsinfo = 0;
 	}
 	else
 	{
@@ -496,7 +497,11 @@ c_topic(struct client *client_p, const char *parv[], int parc)
 				 client_p->user->host);
 		else
 			strlcpy(chptr->topicwho, client_p->name, sizeof(chptr->topicwho));
+
+		chptr->topic_tsinfo = CURRENT_TIME;
 	}
+
+	hook_call(HOOK_CHANNEL_TOPIC, chptr, NULL);
 }
 
 /* c_tb()
@@ -506,6 +511,7 @@ static void
 c_tb(struct client *client_p, const char *parv[], int parc)
 {
 	struct channel *chptr;
+	time_t topicts;
 
 	if(parc < 3 || !IsServer(client_p))
 		return;
@@ -513,6 +519,14 @@ c_tb(struct client *client_p, const char *parv[], int parc)
 	if((chptr = find_channel(parv[0])) == NULL)
 		return;
 
+	topicts = atol(parv[1]);
+
+	/* If we have a topic that is older than the one burst to us, ours
+	 * wins -- otherwise process the topic burst
+	 */
+	if(!EmptyString(chptr->topic) && topicts > chptr->topic_tsinfo)
+		return;
+
 	/* :<server> TB <#channel> <topicts> <topicwho> :<topic> */
 	if(parc == 4)
 	{
@@ -521,6 +535,7 @@ c_tb(struct client *client_p, const char *parv[], int parc)
 
 		strlcpy(chptr->topic, parv[3], sizeof(chptr->topic));
 		strlcpy(chptr->topicwho, parv[2], sizeof(chptr->topicwho));
+		chptr->topic_tsinfo = CURRENT_TIME;
 	}
 	/* :<server> TB <#channel> <topicts> :<topic> */
 	else
@@ -530,7 +545,10 @@ c_tb(struct client *client_p, const char *parv[], int parc)
 
 		strlcpy(chptr->topic, parv[2], sizeof(chptr->topic));
 		strlcpy(chptr->topicwho, client_p->name, sizeof(chptr->topicwho));
+		chptr->topic_tsinfo = CURRENT_TIME;
 	}
+
+	hook_call(HOOK_CHANNEL_TOPIC, chptr, NULL);
 }
 
 /* remove_our_modes()
@@ -617,6 +635,8 @@ chmode_to_string(struct chmode *mode)
 		*p++ = 't';
 	if(mode->mode & MODE_REGONLY)
 		*p++ = 'r';
+	if(mode->mode & MODE_SSLONLY)
+		*p++ = 'S';
 
 	if(mode->limit && mode->key[0])
 	{
@@ -666,6 +686,8 @@ chmode_to_string_simple(struct chmode *mode)
 		*p++ = 't';
 	if(mode->mode & MODE_REGONLY)
 		*p++ = 'r';
+	if(mode->mode & MODE_SSLONLY)
+		*p++ = 'S';
 	if(mode->limit)
 		*p++ = 'l';
 	if(mode->key[0])
@@ -736,7 +758,7 @@ c_sjoin(struct client *client_p, const char *parv[], int parc)
 	/* mode of 0 is sent when someone joins remotely with higher TS. */
 	if(strcmp(parv[2], "0"))
 	{
-		args = parse_simple_mode(&newmode, parv, parc, 2);
+		args = parse_simple_mode(&newmode, parv, parc, 2, 1);
 
 		/* invalid mode */
 		s_assert(args);
@@ -874,7 +896,6 @@ c_join(struct client *client_p, const char *parv[], int parc)
 	dlink_node *next_ptr;
 	time_t newts;
 	int keep_old_modes = 1;
-	int keep_new_modes = 1;
 	int args = 0;
 
 	if(parc < 0 || EmptyString(parv[0]))
@@ -923,15 +944,13 @@ c_join(struct client *client_p, const char *parv[], int parc)
 			chptr->tsinfo = 0;
 		else if(newts < chptr->tsinfo)
 			keep_old_modes = 0;
-		else if(chptr->tsinfo < newts)
-			keep_new_modes = 0;
 	}
 
 	newmode.mode = 0;
 	newmode.key[0] = '\0';
 	newmode.limit = 0;
 
-	args = parse_simple_mode(&newmode, parv, parc, 2);
+	args = parse_simple_mode(&newmode, parv, parc, 2, 1);
 
 	/* invalid mode */
 	s_assert(args);
@@ -958,19 +977,6 @@ c_join(struct client *client_p, const char *parv[], int parc)
 		}
 	}
 
-	if(keep_new_modes)
-	{
-		chptr->mode.mode |= newmode.mode;
-
-		if(!chptr->mode.limit || chptr->mode.limit < newmode.limit)
-			chptr->mode.limit = newmode.limit;
-
-		if(!chptr->mode.key[0] || strcmp(chptr->mode.key, newmode.key) > 0)
-			strlcpy(chptr->mode.key, newmode.key,
-				sizeof(chptr->mode.key));
-
-	}
-
 	/* this must be done after we've updated the modes */
 	if(!keep_old_modes)
 		hook_call(HOOK_SJOIN_LOWERTS, chptr, NULL);
